@@ -1,4 +1,4 @@
-// Copyright 2018-2020 Cargill Incorporated
+// Copyright 2018-2021 Cargill Incorporated
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -22,6 +22,8 @@ use splinter::{
 };
 
 use crate::protocol;
+#[cfg(feature = "authorization")]
+use crate::service::rest_api::SCABBARD_READ_PERMISSION;
 use crate::service::{Scabbard, SERVICE_TYPE};
 
 pub fn make_get_state_at_address_endpoint() -> ServiceEndpoint {
@@ -64,6 +66,8 @@ pub fn make_get_state_at_address_endpoint() -> ServiceEndpoint {
             protocol::SCABBARD_GET_STATE_PROTOCOL_MIN,
             protocol::SCABBARD_PROTOCOL_VERSION,
         ))],
+        #[cfg(feature = "authorization")]
+        permission: SCABBARD_READ_PERMISSION,
     }
 }
 
@@ -74,6 +78,7 @@ mod tests {
     use std::path::PathBuf;
     use std::sync::Mutex;
 
+    use cylinder::{secp256k1::Secp256k1Context, Context};
     use reqwest::{blocking::Client, StatusCode, Url};
     use tempdir::TempDir;
     use transact::{
@@ -82,16 +87,25 @@ mod tests {
             batch::BatchBuilder,
             command::{BytesEntry, Command, SetState},
         },
-        signing::hash::HashSigner,
     };
 
+    #[cfg(feature = "authorization")]
+    use splinter::rest_api::auth::authorization::{
+        AuthorizationHandler, AuthorizationHandlerResult,
+    };
     use splinter::{
-        rest_api::{Resource, RestApiBuilder, RestApiServerError, RestApiShutdownHandle},
+        error::InternalError,
+        rest_api::{
+            auth::{
+                identity::{Identity, IdentityProvider},
+                AuthorizationHeader,
+            },
+            AuthConfig, Resource, RestApiBuilder, RestApiServerError, RestApiShutdownHandle,
+        },
         service::Service,
-        signing::hash::HashVerifier,
     };
 
-    use crate::service::{compute_db_paths, state::ScabbardState, Scabbard};
+    use crate::service::{compute_db_paths, state::ScabbardState, Scabbard, ScabbardVersion};
 
     const MOCK_CIRCUIT_ID: &str = "abcde-01234";
     const MOCK_SERVICE_ID: &str = "ABCD";
@@ -124,16 +138,21 @@ mod tests {
             )
             .expect("Failed to initialize state");
 
-            let signer = HashSigner::default();
+            let signing_context = Secp256k1Context::new();
+            let signer = signing_context.new_signer(signing_context.new_random_private_key());
             let batch = BatchBuilder::new()
                 .with_transactions(vec![
-                    make_command_transaction(&[Command::SetState(SetState::new(vec![
-                        BytesEntry::new(address.clone(), value.clone()),
-                    ]))])
+                    make_command_transaction(
+                        &[Command::SetState(SetState::new(vec![BytesEntry::new(
+                            address.clone(),
+                            value.clone(),
+                        )]))],
+                        &*signer,
+                    )
                     .take()
                     .0,
                 ])
-                .build_pair(&signer)
+                .build_pair(&*signer)
                 .expect("Failed to build batch");
             state
                 .prepare_change(batch)
@@ -145,12 +164,13 @@ mod tests {
         let scabbard = Scabbard::new(
             MOCK_SERVICE_ID.into(),
             MOCK_CIRCUIT_ID,
+            ScabbardVersion::V1,
             Default::default(),
             paths.temp_dir.path(),
             TEMP_DB_SIZE,
             paths.temp_dir.path(),
             TEMP_DB_SIZE,
-            Box::new(HashVerifier),
+            Secp256k1Context::new().new_verifier(),
             vec![],
             None,
         )
@@ -173,6 +193,7 @@ mod tests {
                 "SplinterProtocolVersion",
                 protocol::SCABBARD_PROTOCOL_VERSION,
             )
+            .header("Authorization", "test")
             .send()
             .expect("Failed to perform request");
         assert_eq!(resp.status(), StatusCode::NOT_FOUND);
@@ -185,6 +206,7 @@ mod tests {
                 "SplinterProtocolVersion",
                 protocol::SCABBARD_PROTOCOL_VERSION,
             )
+            .header("Authorization", "test")
             .send()
             .expect("Failed to perform request");
         assert_eq!(resp.status(), StatusCode::OK);
@@ -232,13 +254,30 @@ mod tests {
             resource = resource.add_request_guard(request_guard);
         }
         let handler = service_endpoint.handler;
-        resource.add_method(service_endpoint.method, move |request, payload| {
-            (handler)(
-                request,
-                payload,
-                &*service.lock().expect("Service lock poisoned"),
+        #[cfg(feature = "authorization")]
+        {
+            resource.add_method(
+                service_endpoint.method,
+                service_endpoint.permission,
+                move |request, payload| {
+                    (handler)(
+                        request,
+                        payload,
+                        &*service.lock().expect("Service lock poisoned"),
+                    )
+                },
             )
-        })
+        }
+        #[cfg(not(feature = "authorization"))]
+        {
+            resource.add_method(service_endpoint.method, move |request, payload| {
+                (handler)(
+                    request,
+                    payload,
+                    &*service.lock().expect("Service lock poisoned"),
+                )
+            })
+        }
     }
 
     fn run_rest_api_on_open_port(
@@ -247,9 +286,17 @@ mod tests {
         (10000..20000)
             .find_map(|port| {
                 let bind_url = format!("127.0.0.1:{}", port);
-                let result = RestApiBuilder::new()
+                let rest_api_builder = RestApiBuilder::new()
                     .with_bind(&bind_url)
                     .add_resources(resources.clone())
+                    .with_auth_configs(vec![AuthConfig::Custom {
+                        resources: vec![],
+                        identity_provider: Box::new(AlwaysAcceptIdentityProvider),
+                    }]);
+                #[cfg(feature = "authorization")]
+                let rest_api_builder = rest_api_builder
+                    .with_authorization_handlers(vec![Box::new(AlwaysAllowAuthorizationHandler)]);
+                let result = rest_api_builder
                     .build()
                     .expect("Failed to build REST API")
                     .run();
@@ -263,4 +310,41 @@ mod tests {
             })
             .expect("No port available")
     }
+
+    /// An identity provider that always returns `Ok(Some(_))`
+    #[derive(Clone)]
+    struct AlwaysAcceptIdentityProvider;
+
+    impl IdentityProvider for AlwaysAcceptIdentityProvider {
+        fn get_identity(
+            &self,
+            _authorization: &AuthorizationHeader,
+        ) -> Result<Option<Identity>, InternalError> {
+            Ok(Some(Identity::Custom("identity".into())))
+        }
+
+        fn clone_box(&self) -> Box<dyn IdentityProvider> {
+            Box::new(self.clone())
+        }
+    }
+
+    /// An authorization handler that always returns `Ok(AuthorizationHandlerResult::Allow)`
+    #[cfg(feature = "authorization")]
+    #[derive(Clone)]
+    struct AlwaysAllowAuthorizationHandler;
+
+    #[cfg(feature = "authorization")]
+    impl AuthorizationHandler for AlwaysAllowAuthorizationHandler {
+        fn has_permission(
+            &self,
+            _identity: &Identity,
+            _permission_id: &str,
+        ) -> Result<AuthorizationHandlerResult, InternalError> {
+            Ok(AuthorizationHandlerResult::Allow)
+        }
+
+        fn clone_box(&self) -> Box<dyn AuthorizationHandler> {
+            Box::new(self.clone())
+        }
+    }
 }
